<?php

/**
 * @file
 * Contains \Drupal\ontology\Controller\OntologyController.
 */

namespace Drupal\ontology\Controller;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Response;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;

// XML Namespaces.
define('OWL_NS_RDF', 'http://www.w3.org/1999/02/22-rdf-syntax-ns#');
define('OWL_NS_OWL', 'http://www.w3.org/2002/07/owl#');
define('OWL_NS_XSD', 'http://www.w3.org/2001/XMLSchema#');
define('OWL_NS_RDFS', 'http://www.w3.org/2000/01/rdf-schema#');

class OntologyController extends ControllerBase {
 
  /**
   * OWL ontology callback.
   */
  public function owl() {
    // Ontology URL.
    $url = \Drupal::request()->getRequestUri();

    // XML header, doctype, entities and root node.
    $xml = '<?xml version="1.0" encoding="UTF-8"?>';
    $xml = "\r\n<!DOCTYPE rdf:RDF [\r\n";
    foreach (rdf_get_namespaces() as $prefix => $namespace) {
      $xml .= "    <!ENTITY $prefix \"$namespace\" >\r\n";
    }
    $xml .= "\r\n]>\r\n<rdf:RDF xmlns:rdf=\"" . OWL_NS_RDF . "\"></rdf:RDF>";

    $doc = new \DOMDocument('1.0', 'UTF-8');
    $doc->loadXML($xml);
    $doc->preserveWhiteSpace = FALSE;
    $doc->formatOutput = TRUE;

    // Add namespaces to root node.
    $rdf = $doc->documentElement;
    $rdf->setAttribute('xml:base', $url);
    foreach (rdf_get_namespaces() as $prefix => $namespace) {
      $rdf->setAttributeNS('http://www.w3.org/2000/xmlns/', 'xmlns:' . $prefix, $namespace);
    }

    // Add ontology.
    $ontology = $rdf->appendChild($doc->createElementNS(OWL_NS_OWL, 'Ontology'));
    $ontology->setAttributeNS(OWL_NS_RDF, 'about', $url);

    // Entity types.
    $rdf->appendChild($doc->createComment(' Classes for Drupal entities with RDF mapping '));
    
    $entity_types = \Drupal::service('entity_type.bundle.info')->getAllBundleInfo();
    foreach ($entity_types as $entity_type => $bundles) {
      // Add OWL class for each entity type.
      $rdf->appendChild($doc->createComment(" Classes for {$entity_type} entities "));
      $rdf->appendChild($doc->createComment(" $url#$entity_type "));
      $class = $rdf->appendChild($doc->createElementNS(OWL_NS_OWL, 'Class'));
      $class->setAttributeNS(OWL_NS_RDF, 'about', "#$entity_type");

      // Add subclass for each bundle.
      foreach ($bundles as $bundle => $bundle_info) {
        // If single bundle with the same name as entity type.
        if ($bundle == $entity_type) {
          // Add human readable name as label.
          $class->appendChild($doc->createElementNS(OWL_NS_RDFS, 'label', $bundle_info['label']));
        }
        else {
          // Add OWL class for each bundle.
          $rdf->appendChild($doc->createComment(" $url#$bundle "));
          $subclass = $rdf->appendChild($doc->createElementNS(OWL_NS_OWL, 'Class'));
          $subclass->setAttributeNS(OWL_NS_RDF, 'about', "#$bundle");
          $subclass->appendChild($doc->createElementNS(OWL_NS_RDFS, 'label', $bundle_info['label']));
          $subclass_of = $subclass->appendChild($doc->createElementNS(OWL_NS_RDFS, 'subClassOf'));
          $subclass_of->setAttributeNS(OWL_NS_RDF, 'resource', "#$entity_type");
          if ($entity_type == 'commerce_product_variation') {
             $subclass_of = $subclass->appendChild($doc->createElementNS(OWL_NS_RDFS, 'subClassOf'));
             $subclass_of->setAttributeNS(OWL_NS_RDF, 'resource', "http://schema.org/Offer");
             //<rdfs:subClassOf rdf:resource="http://schema.org/Product"/>
          }
        }
        $rdf_mapping = rdf_get_mapping($entity_type, $bundle);
        //print_r($rdf_mapping->toArray());
        // TODO Set subclasses from RDF mapping.        
      }      
    }
    
    // Return XML response.
    $response = new Response();
    $response->setContent($doc->saveXML());
    return $response;
  }
 
} 
